@@ -37,106 +37,16 @@
 #include "cpl_string.h"
 #include "gdal.h"
 
-/* -------------------------------------------------------------------- */
-/*                   DoesDriverHandleExtension()                        */
-/* -------------------------------------------------------------------- */
-
-static bool DoesDriverHandleExtension(GDALDriverH hDriver, const char *pszExt)
-{
-    bool bRet = false;
-    const char *pszDriverExtensions =
-        GDALGetMetadataItem(hDriver, GDAL_DMD_EXTENSIONS, nullptr);
-    if (pszDriverExtensions)
-    {
-        char **papszTokens = CSLTokenizeString(pszDriverExtensions);
-        for (int j = 0; papszTokens[j]; j++)
-        {
-            if (EQUAL(pszExt, papszTokens[j]))
-            {
-                bRet = true;
-                break;
-            }
-        }
-        CSLDestroy(papszTokens);
-    }
-    return bRet;
-}
-
 /* -------------------------------------------------------------------- */
 /*                         GetOutputDriversFor()                        */
 /* -------------------------------------------------------------------- */
 
-std::vector<CPLString> GetOutputDriversFor(const char *pszDestFilename,
-                                           int nFlagRasterVector)
+std::vector<std::string> GetOutputDriversFor(const char *pszDestFilename,
+                                             int nFlagRasterVector)
 {
-    std::vector<CPLString> aoDriverList;
-
-    CPLString osExt = CPLGetExtension(pszDestFilename);
-    if (EQUAL(osExt, "zip") &&
-        (CPLString(pszDestFilename).endsWith(".shp.zip") ||
-         CPLString(pszDestFilename).endsWith(".SHP.ZIP")))
-    {
-        osExt = "shp.zip";
-    }
-    else if (EQUAL(osExt, "zip") &&
-             (CPLString(pszDestFilename).endsWith(".gpkg.zip") ||
-              CPLString(pszDestFilename).endsWith(".GPKG.ZIP")))
-    {
-        osExt = "gpkg.zip";
-    }
-    const int nDriverCount = GDALGetDriverCount();
-    for (int i = 0; i < nDriverCount; i++)
-    {
-        GDALDriverH hDriver = GDALGetDriver(i);
-        bool bOk = false;
-        if ((GDALGetMetadataItem(hDriver, GDAL_DCAP_CREATE, nullptr) !=
-                 nullptr ||
-             GDALGetMetadataItem(hDriver, GDAL_DCAP_CREATECOPY, nullptr) !=
-                 nullptr) &&
-            (((nFlagRasterVector & GDAL_OF_RASTER) &&
-              GDALGetMetadataItem(hDriver, GDAL_DCAP_RASTER, nullptr) !=
-                  nullptr) ||
-             ((nFlagRasterVector & GDAL_OF_VECTOR) &&
-              GDALGetMetadataItem(hDriver, GDAL_DCAP_VECTOR, nullptr) !=
-                  nullptr)))
-        {
-            bOk = true;
-        }
-        else if (GDALGetMetadataItem(hDriver, GDAL_DCAP_VECTOR_TRANSLATE_FROM,
-                                     nullptr) &&
-                 (nFlagRasterVector & GDAL_OF_VECTOR) != 0)
-        {
-            bOk = true;
-        }
-        if (bOk)
-        {
-            if (!osExt.empty() && DoesDriverHandleExtension(hDriver, osExt))
-            {
-                aoDriverList.push_back(GDALGetDriverShortName(hDriver));
-            }
-            else
-            {
-                const char *pszPrefix = GDALGetMetadataItem(
-                    hDriver, GDAL_DMD_CONNECTION_PREFIX, nullptr);
-                if (pszPrefix && STARTS_WITH_CI(pszDestFilename, pszPrefix))
-                {
-                    aoDriverList.push_back(GDALGetDriverShortName(hDriver));
-                }
-            }
-        }
-    }
-
-    // GMT is registered before netCDF for opening reasons, but we want
-    // netCDF to be used by default for output.
-    if (EQUAL(osExt, "nc") && aoDriverList.size() == 2 &&
-        EQUAL(aoDriverList[0], "GMT") && EQUAL(aoDriverList[1], "NETCDF"))
-    {
-        aoDriverList.clear();
-        aoDriverList.push_back("NETCDF");
-        aoDriverList.push_back("GMT");
-    }
-
-    return aoDriverList;
+    return CPLStringList(GDALGetOutputDriversForDatasetName(
+        pszDestFilename, nFlagRasterVector, /* bSingleMatch = */ false,
+        /* bEmitWarning = */ false));
 }
 
 /* -------------------------------------------------------------------- */
@@ -145,36 +55,15 @@ std::vector<CPLString> GetOutputDriversFor(const char *pszDestFilename,
 
 CPLString GetOutputDriverForRaster(const char *pszDestFilename)
 {
-    CPLString osFormat;
-    std::vector<CPLString> aoDrivers =
-        GetOutputDriversFor(pszDestFilename, GDAL_OF_RASTER);
-    CPLString osExt(CPLGetExtension(pszDestFilename));
-    if (aoDrivers.empty())
+    const CPLStringList aosList(GDALGetOutputDriversForDatasetName(
+        pszDestFilename, GDAL_OF_RASTER, /* bSingleMatch = */ true,
+        /* bEmitWarning = */ true));
+    if (!aosList.empty())
     {
-        if (osExt.empty())
-        {
-            osFormat = "GTiff";
-        }
-        else
-        {
-            CPLError(CE_Failure, CPLE_AppDefined, "Cannot guess driver for %s",
-                     pszDestFilename);
-            return "";
-        }
+        CPLDebug("GDAL", "Using %s driver", aosList[0]);
+        return aosList[0];
     }
-    else
-    {
-        if (aoDrivers.size() > 1 &&
-            !(aoDrivers[0] == "GTiff" && aoDrivers[1] == "COG"))
-        {
-            CPLError(CE_Warning, CPLE_AppDefined,
-                     "Several drivers matching %s extension. Using %s",
-                     osExt.c_str(), aoDrivers[0].c_str());
-        }
-        osFormat = aoDrivers[0];
-    }
-    CPLDebug("GDAL", "Using %s driver", osFormat.c_str());
-    return osFormat;
+    return CPLString();
 }
 
 /* -------------------------------------------------------------------- */
@@ -223,12 +112,11 @@ void GDALRemoveBOM(GByte *pabyData)
 
 std::string GDALRemoveSQLComments(const std::string &osInput)
 {
-    char **papszLines =
-        CSLTokenizeStringComplex(osInput.c_str(), "\r\n", FALSE, FALSE);
+    const CPLStringList aosLines(
+        CSLTokenizeStringComplex(osInput.c_str(), "\r\n", FALSE, FALSE));
     std::string osSQL;
-    for (char **papszIter = papszLines; papszIter && *papszIter; ++papszIter)
+    for (const char *pszLine : aosLines)
     {
-        const char *pszLine = *papszIter;
         char chQuote = 0;
         int i = 0;
         for (; pszLine[i] != '\0'; ++i)
@@ -262,6 +150,15 @@ std::string GDALRemoveSQLComments(const std::string &osInput)
         }
         osSQL += ' ';
     }
-    CSLDestroy(papszLines);
     return osSQL;
 }
+
+/************************************************************************/
+/*                            ArgIsNumeric()                            */
+/************************************************************************/
+
+int ArgIsNumeric(const char *pszArg)
+
+{
+    return CPLGetValueType(pszArg) != CPL_VALUE_STRING;
+}
