@@ -225,7 +225,7 @@ int GTiffRasterBand::DirectIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
         panOffsets[iLine] +=
             (nXOff + static_cast<vsi_l_offset>(nYOffsetInBlock) * nBlockXSize) *
             nSrcPixelSize;
-        panSizes[iLine] = nReqXSize * nSrcPixelSize;
+        panSizes[iLine] = static_cast<size_t>(nReqXSize) * nSrcPixelSize;
     }
 
     // Extract data from the file.
@@ -615,7 +615,6 @@ CPLVirtualMem *GTiffRasterBand::GetVirtualMemAutoInternal(GDALRWFlag eRWFlag,
 /*                         CacheMultiRange()                            */
 /************************************************************************/
 
-#ifdef SUPPORTS_GET_OFFSET_BYTECOUNT
 static bool CheckTrailer(const GByte *strileData, vsi_l_offset nStrileSize)
 {
     GByte abyTrailer[4];
@@ -632,7 +631,6 @@ static bool CheckTrailer(const GByte *strileData, vsi_l_offset nStrileSize)
     }
     return memcmp(abyTrailer, abyLastBytes, 4) == 0;
 }
-#endif
 
 void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
                                        int nYSize, int nBufXSize, int nBufYSize,
@@ -670,14 +668,16 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
             std::min(static_cast<double>(nRasterYSize - 1),
                      (nBufYSize - 1 + 0.5) * dfSrcYInc + dfYOff + EPS)) /
         nBlockYSize;
-#ifdef SUPPORTS_GET_OFFSET_BYTECOUNT
+
     const int nBlockCount = nBlocksPerRow * nBlocksPerColumn;
+
     struct StrileData
     {
         vsi_l_offset nOffset;
         vsi_l_offset nByteCount;
         bool bTryMask;
     };
+
     std::map<int, StrileData> oMapStrileToOffsetByteCount;
 
     // Dedicated method to retrieved the offset and size in an efficient way
@@ -792,7 +792,7 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
             {
                 // Sparse tile
                 m_poGDS->m_oCacheStrileToOffsetByteCount.insert(
-                    nBlockId, std::pair<vsi_l_offset, vsi_l_offset>(0, 0));
+                    nBlockId, std::pair(0, 0));
                 continue;
             }
 
@@ -855,8 +855,7 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
                          nBlockId, nRealOffset, nRealSize);
 #endif
                 m_poGDS->m_oCacheStrileToOffsetByteCount.insert(
-                    nBlockId, std::pair<vsi_l_offset, vsi_l_offset>(nRealOffset,
-                                                                    nRealSize));
+                    nBlockId, std::pair(nRealOffset, nRealSize));
             }
 
             // Processing of mask
@@ -915,8 +914,7 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
 #endif
 
                     m_poGDS->m_poMaskDS->m_oCacheStrileToOffsetByteCount.insert(
-                        nBlockId, std::pair<vsi_l_offset, vsi_l_offset>(
-                                      nRealOffset, nRealSize));
+                        nBlockId, std::pair(nRealOffset, nRealSize));
                 }
             }
             if (!bOK)
@@ -929,7 +927,6 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
         }
         return true;
     };
-#endif
 
     thandle_t th = TIFFClientdata(m_poGDS->m_hTIFF);
     if (!VSI_TIFFHasCachedRanges(th))
@@ -955,7 +952,6 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
                 vsi_l_offset nOffset = 0;
                 vsi_l_offset nSize = 0;
 
-#ifdef SUPPORTS_GET_OFFSET_BYTECOUNT
                 if ((m_poGDS->m_nPlanarConfig == PLANARCONFIG_CONTIG ||
                      m_poGDS->nBands == 1) &&
                     !m_poGDS->m_bStreamingIn &&
@@ -967,7 +963,6 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
                                                    nMaxRawBlockCacheSize);
                 }
                 else
-#endif
                 {
                     CPL_IGNORE_RET_VAL(
                         m_poGDS->IsBlockAvailable(nBlockId, &nOffset, &nSize));
@@ -983,8 +978,8 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
                                  iX, iY, nOffset,
                                  nOffset + static_cast<size_t>(nSize) - 1);
 #endif
-                        aOffsetSize.push_back(std::pair<vsi_l_offset, size_t>(
-                            nOffset, static_cast<size_t>(nSize)));
+                        aOffsetSize.push_back(
+                            std::pair(nOffset, static_cast<size_t>(nSize)));
                         nTotalSize += static_cast<size_t>(nSize);
                     }
                     else
@@ -1061,7 +1056,6 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
                                         &apData[0], &anOffsets[0], &anSizes[0],
                                         fp) == 0)
                 {
-#ifdef SUPPORTS_GET_OFFSET_BYTECOUNT
                     if (!oMapStrileToOffsetByteCount.empty() &&
                         !FillCacheStrileToOffsetByteCount(anOffsets, anSizes,
                                                           apData))
@@ -1076,7 +1070,6 @@ void *GTiffRasterBand::CacheMultiRange(int nXOff, int nYOff, int nXSize,
                         return pRet;
                     }
 
-#endif
                     VSI_TIFFSetCachedRanges(
                         th, static_cast<int>(anSizes.size()), &apData[0],
                         &anOffsets[0], &anSizes[0]);
@@ -1348,9 +1341,8 @@ CPLErr GTiffRasterBand::IReadBlock(int nBlockXOff, int nBlockYOff, void *pImage)
         }
     }
 
-#ifdef SUPPORTS_GET_OFFSET_BYTECOUNT
     CacheMaskForBlock(nBlockXOff, nBlockYOff);
-#endif
+
     return eErr;
 }
 
@@ -1358,7 +1350,6 @@ CPLErr GTiffRasterBand::IReadBlock(int nBlockXOff, int nBlockYOff, void *pImage)
 /*                           CacheMaskForBlock()                       */
 /************************************************************************/
 
-#ifdef SUPPORTS_GET_OFFSET_BYTECOUNT
 void GTiffRasterBand::CacheMaskForBlock(int nBlockXOff, int nBlockYOff)
 
 {
@@ -1378,7 +1369,6 @@ void GTiffRasterBand::CacheMaskForBlock(int nBlockXOff, int nBlockYOff)
         }
     }
 }
-#endif
 
 /************************************************************************/
 /*                       FillCacheForOtherBands()                       */
@@ -1604,7 +1594,7 @@ const char *GTiffRasterBand::GetMetadataItem(const char *pszName,
             return CPLSPrintf(CPL_FRMT_GUIB, static_cast<GUIntBig>(nByteCount));
         }
     }
-    else if (pszDomain != nullptr && EQUAL(pszDomain, "_DEBUG_"))
+    else if (pszName && pszDomain && EQUAL(pszDomain, "_DEBUG_"))
     {
         if (EQUAL(pszName, "HAS_BLOCK_CACHE"))
             return HasBlockCache() ? "1" : "0";
@@ -1612,7 +1602,7 @@ const char *GTiffRasterBand::GetMetadataItem(const char *pszName,
 
     const char *pszRet = m_oGTiffMDMD.GetMetadataItem(pszName, pszDomain);
 
-    if (pszRet == nullptr && eDataType == GDT_Byte && pszDomain != nullptr &&
+    if (pszRet == nullptr && eDataType == GDT_Byte && pszName && pszDomain &&
         EQUAL(pszDomain, "IMAGE_STRUCTURE") && EQUAL(pszName, "PIXELTYPE"))
     {
         // to get a chance of emitting the warning about this legacy usage
