@@ -40,12 +40,6 @@
 
 using namespace PCIDSK;
 
-/* -------------------------------------------------------------------- */
-/*      Size of a block in the record/vertex block tables.  This is    */
-/*      determined by the PCIDSK format and may not be changed.         */
-/* -------------------------------------------------------------------- */
-static const int block_page_size = 8192;
-
 /************************************************************************/
 /*                            VecSegHeader()                            */
 /************************************************************************/
