@@ -27,14 +27,11 @@
  ****************************************************************************/
 
 #include <cassert>
+#include <limits>
 
 #include "tiledbheaders.h"
 
-#ifdef _MSC_VER
-#pragma warning(push)
-// 'tiledb::Array::Array': was declared deprecated
-#pragma warning(disable : 4996) /* XXXX was deprecated */
-#endif
+constexpr const char *RASTER_DATASET_TYPE = "raster";
 
 /************************************************************************/
 /* ==================================================================== */
@@ -52,7 +49,8 @@ class TileDBRasterBand final : public GDALPamRasterBand
     CPLString osAttrName;
 
   public:
-    TileDBRasterBand(TileDBRasterDataset *, int, CPLString = TILEDB_VALUES);
+    TileDBRasterBand(TileDBRasterDataset *, int,
+                     const std::string &osAttr = TILEDB_VALUES);
     virtual CPLErr IReadBlock(int, int, void *) override;
     virtual CPLErr IWriteBlock(int, int, void *) override;
     virtual CPLErr IRasterIO(GDALRWFlag, int, int, int, int, void *, int, int,
@@ -109,8 +107,6 @@ static const char *index_type_name(TILEDB_INTERLEAVE_MODE eMode)
 /*                             SetBuffer()                              */
 /************************************************************************/
 
-#if ((TILEDB_VERSION_MAJOR > 2) ||                                             \
-     (TILEDB_VERSION_MAJOR == 2 && TILEDB_VERSION_MINOR >= 4))
 static CPLErr SetBuffer(tiledb::Query *poQuery, GDALDataType eType,
                         const CPLString &osAttrName, void *pImage, int nSize)
 {
@@ -120,6 +116,10 @@ static CPLErr SetBuffer(tiledb::Query *poQuery, GDALDataType eType,
             poQuery->set_data_buffer(
                 osAttrName, reinterpret_cast<unsigned char *>(pImage), nSize);
             break;
+        case GDT_Int8:
+            poQuery->set_data_buffer(osAttrName,
+                                     reinterpret_cast<int8_t *>(pImage), nSize);
+            break;
         case GDT_UInt16:
             poQuery->set_data_buffer(
                 osAttrName, reinterpret_cast<unsigned short *>(pImage), nSize);
@@ -128,6 +128,10 @@ static CPLErr SetBuffer(tiledb::Query *poQuery, GDALDataType eType,
             poQuery->set_data_buffer(
                 osAttrName, reinterpret_cast<unsigned int *>(pImage), nSize);
             break;
+        case GDT_UInt64:
+            poQuery->set_data_buffer(
+                osAttrName, reinterpret_cast<uint64_t *>(pImage), nSize);
+            break;
         case GDT_Int16:
             poQuery->set_data_buffer(osAttrName,
                                      reinterpret_cast<short *>(pImage), nSize);
@@ -136,6 +140,10 @@ static CPLErr SetBuffer(tiledb::Query *poQuery, GDALDataType eType,
             poQuery->set_data_buffer(osAttrName,
                                      reinterpret_cast<int *>(pImage), nSize);
             break;
+        case GDT_Int64:
+            poQuery->set_data_buffer(
+                osAttrName, reinterpret_cast<int64_t *>(pImage), nSize);
+            break;
         case GDT_Float32:
             poQuery->set_data_buffer(osAttrName,
                                      reinterpret_cast<float *>(pImage), nSize);
@@ -165,74 +173,76 @@ static CPLErr SetBuffer(tiledb::Query *poQuery, GDALDataType eType,
     }
     return CE_None;
 }
-#else
-static CPLErr SetBuffer(tiledb::Query *poQuery, GDALDataType eType,
-                        const CPLString &osAttrName, void *pImage, int nSize)
-{
-    switch (eType)
-    {
-        case GDT_Byte:
-            poQuery->set_buffer(
-                osAttrName, reinterpret_cast<unsigned char *>(pImage), nSize);
-            break;
-        case GDT_UInt16:
-            poQuery->set_buffer(
-                osAttrName, reinterpret_cast<unsigned short *>(pImage), nSize);
-            break;
-        case GDT_UInt32:
-            poQuery->set_buffer(
-                osAttrName, reinterpret_cast<unsigned int *>(pImage), nSize);
-            break;
-        case GDT_Int16:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<short *>(pImage),
-                                nSize);
-            break;
-        case GDT_Int32:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<int *>(pImage),
-                                nSize);
-            break;
-        case GDT_Float32:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<float *>(pImage),
-                                nSize);
-            break;
-        case GDT_Float64:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<double *>(pImage),
-                                nSize);
-            break;
-        case GDT_CInt16:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<short *>(pImage),
-                                nSize * 2);
-            break;
-        case GDT_CInt32:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<int *>(pImage),
-                                nSize * 2);
-            break;
-        case GDT_CFloat32:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<float *>(pImage),
-                                nSize * 2);
-            break;
-        case GDT_CFloat64:
-            poQuery->set_buffer(osAttrName, reinterpret_cast<double *>(pImage),
-                                nSize * 2);
-            break;
-        default:
-            return CE_Failure;
-    }
-    return CE_None;
-}
-#endif
 
 /************************************************************************/
 /*                          TileDBRasterBand()                          */
 /************************************************************************/
 
 TileDBRasterBand::TileDBRasterBand(TileDBRasterDataset *poDSIn, int nBandIn,
-                                   CPLString osAttr)
+                                   const std::string &osAttr)
     : poGDS(poDSIn), bStats(poDSIn->bStats), osAttrName(osAttr)
 {
     poDS = poDSIn;
     nBand = nBandIn;
     eDataType = poGDS->eDataType;
+    if (eDataType == GDT_Unknown)
+    {
+        try
+        {
+            auto attr = (poGDS->m_roArray ? poGDS->m_roArray : poGDS->m_array)
+                            ->schema()
+                            .attribute(osAttr);
+            switch (attr.type())
+            {
+                case TILEDB_INT8:
+                    eDataType = GDT_Int8;
+                    break;
+                case TILEDB_UINT8:
+                    eDataType = GDT_Byte;
+                    break;
+                case TILEDB_INT16:
+                    eDataType =
+                        attr.cell_val_num() == 2 ? GDT_CInt16 : GDT_Int16;
+                    break;
+                case TILEDB_UINT16:
+                    eDataType = GDT_UInt16;
+                    break;
+                case TILEDB_INT32:
+                    eDataType =
+                        attr.cell_val_num() == 2 ? GDT_CInt32 : GDT_Int32;
+                    break;
+                case TILEDB_UINT32:
+                    eDataType = GDT_UInt32;
+                    break;
+                case TILEDB_INT64:
+                    eDataType = GDT_Int64;
+                    break;
+                case TILEDB_UINT64:
+                    eDataType = GDT_UInt64;
+                    break;
+                case TILEDB_FLOAT32:
+                    eDataType =
+                        attr.cell_val_num() == 2 ? GDT_CFloat32 : GDT_Float32;
+                    break;
+                case TILEDB_FLOAT64:
+                    eDataType =
+                        attr.cell_val_num() == 2 ? GDT_CFloat64 : GDT_Float64;
+                    break;
+                default:
+                {
+                    const char *pszTypeName = "";
+                    tiledb_datatype_to_str(attr.type(), &pszTypeName);
+                    CPLError(CE_Failure, CPLE_NotSupported,
+                             "Unhandled TileDB data type: %s", pszTypeName);
+                    break;
+                }
+            }
+        }
+        catch (const tiledb::TileDBError &e)
+        {
+            CPLError(CE_Failure, CPLE_AppDefined, "%s", e.what());
+        }
+    }
     eAccess = poGDS->eAccess;
     nRasterXSize = poGDS->nRasterXSize;
     nRasterYSize = poGDS->nRasterYSize;
@@ -265,36 +275,34 @@ CPLErr TileDBRasterBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
         nBufferDTSize > 0 && (nPixelSpace % nBufferDTSize) == 0 &&
         (nLineSpace % nBufferDTSize) == 0)
     {
-        std::unique_ptr<tiledb::Query> poQuery;
-        int nBandIdx = poGDS->nBandStart + nBand - 1;
+        const uint64_t nBandIdx = poGDS->nBandStart + nBand - 1;
         std::vector<uint64_t> oaSubarray = {
-            uint64_t(nBandIdx), uint64_t(nBandIdx),
-            (uint64_t)nYOff,    (uint64_t)nYOff + nYSize - 1,
-            (uint64_t)nXOff,    (uint64_t)nXOff + nXSize - 1};
+            nBandIdx,        nBandIdx,
+            (uint64_t)nYOff, (uint64_t)nYOff + nYSize - 1,
+            (uint64_t)nXOff, (uint64_t)nXOff + nXSize - 1};
         if (poGDS->eIndexMode == PIXEL)
             std::rotate(oaSubarray.begin(), oaSubarray.begin() + 2,
                         oaSubarray.end());
 
-        if ((eRWFlag == GF_Read) &&
-            ((eAccess == GA_Update) && (poGDS->m_roArray)))
-        {
-            poQuery.reset(
-                new tiledb::Query(*poGDS->m_roCtx, *poGDS->m_roArray));
-        }
-        else
-        {
-            poQuery.reset(new tiledb::Query(*poGDS->m_ctx, *poGDS->m_array));
-        }
+        const bool bUseReadOnlyObjs =
+            ((eRWFlag == GF_Read) && (eAccess == GA_Update) &&
+             (poGDS->m_roArray));
+        const auto &oCtxt = bUseReadOnlyObjs ? *poGDS->m_roCtx : *poGDS->m_ctx;
+        const auto &oArray =
+            bUseReadOnlyObjs ? *poGDS->m_roArray : *poGDS->m_array;
 
+        auto poQuery = std::make_unique<tiledb::Query>(oCtxt, oArray);
+        tiledb::Subarray subarray(oCtxt, oArray);
         if (poGDS->m_array->schema().domain().ndim() == 3)
         {
-            poQuery->set_subarray(oaSubarray);
+            subarray.set_subarray(oaSubarray);
         }
         else
         {
-            poQuery->set_subarray(std::vector<uint64_t>(oaSubarray.cbegin() + 2,
+            subarray.set_subarray(std::vector<uint64_t>(oaSubarray.cbegin() + 2,
                                                         oaSubarray.cend()));
         }
+        poQuery->set_subarray(subarray);
 
         SetBuffer(poQuery.get(), eDataType, osAttrName, pData, nXSize * nYSize);
 
@@ -305,7 +313,7 @@ CPLErr TileDBRasterBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
         {
             for (auto const &poAttrDS : poGDS->lpoAttributeDS)
             {
-                GDALRasterBand *poAttrBand = poAttrDS->GetRasterBand(nBandIdx);
+                GDALRasterBand *poAttrBand = poAttrDS->GetRasterBand(nBand);
                 GDALDataType eAttrType = poAttrBand->GetRasterDataType();
                 int nBytes = GDALGetDataTypeSizeBytes(eAttrType);
                 size_t nValues = static_cast<size_t>(nBufXSize) * nBufYSize;
@@ -474,50 +482,44 @@ CPLErr TileDBRasterDataset::IRasterIO(
         nBufferDTSize > 0 && (nPixelSpace % nBufferDTSize) == 0 &&
         (nLineSpace % nBufferDTSize) == 0)
     {
-        std::unique_ptr<tiledb::Query> poQuery;
         std::vector<uint64_t> oaSubarray = {
             (uint64_t)nYOff, (uint64_t)nYOff + nYSize - 1, (uint64_t)nXOff,
             (uint64_t)nXOff + nXSize - 1};
 
-        if ((eRWFlag == GF_Read) && (eAccess == GA_Update && m_roArray))
+        const bool bUseReadOnlyObjs =
+            ((eRWFlag == GF_Read) && (eAccess == GA_Update) && (m_roArray));
+        const auto &oCtxt = bUseReadOnlyObjs ? *m_roCtx : *m_ctx;
+        const auto &oArray = bUseReadOnlyObjs ? *m_roArray : *m_array;
+
+        auto poQuery = std::make_unique<tiledb::Query>(oCtxt, oArray);
+        tiledb::Subarray subarray(oCtxt, oArray);
+        subarray.set_subarray(oaSubarray);
+        poQuery->set_subarray(subarray);
+
+        for (int b = 0; b < nBandCount; b++)
         {
-            poQuery.reset(new tiledb::Query(*m_roCtx, *m_roArray));
+            TileDBRasterBand *poBand =
+                (TileDBRasterBand *)GetRasterBand(panBandMap[b]);
+            int nRegionSize = nBufXSize * nBufYSize * nBufferDTSize;
+            SetBuffer(poQuery.get(), eDataType, poBand->osAttrName,
+                      ((GByte *)pData) + b * nRegionSize, nRegionSize);
         }
+
+        if (bStats)
+            tiledb::Stats::enable();
+
+        auto status = poQuery->submit();
+
+        if (bStats)
+        {
+            tiledb::Stats::dump(stdout);
+            tiledb::Stats::disable();
+        }
+
+        if (status == tiledb::Query::Status::FAILED)
+            return CE_Failure;
         else
-        {
-            poQuery.reset(new tiledb::Query(*m_ctx, *m_array));
-        }
-
-        if (poQuery != nullptr)
-        {
-            poQuery->set_subarray(oaSubarray);
-
-            for (int b = 0; b < nBandCount; b++)
-            {
-                TileDBRasterBand *poBand =
-                    (TileDBRasterBand *)GetRasterBand(panBandMap[b]);
-                int nRegionSize = nBufXSize * nBufYSize * nBufferDTSize;
-                SetBuffer(poQuery.get(), eDataType, poBand->osAttrName,
-                          ((GByte *)pData) + b * nRegionSize, nRegionSize);
-            }
-
-            if (bStats)
-                tiledb::Stats::enable();
-
-            auto status = poQuery->submit();
-
-            if (bStats)
-            {
-                tiledb::Stats::dump(stdout);
-                tiledb::Stats::disable();
-            }
-
-            if (status == tiledb::Query::Status::FAILED)
-                return CE_Failure;
-            else
-                return CE_None;
-        }
-        return CE_Failure;
+            return CE_None;
     }
 
     return GDALPamDataset::IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize,
@@ -531,6 +533,7 @@ CPLErr TileDBRasterDataset::IRasterIO(
 /************************************************************************/
 
 CPLErr TileDBRasterDataset::AddDimensions(tiledb::Domain &domain,
+                                          const char *pszAttrName,
                                           tiledb::Dimension &y,
                                           tiledb::Dimension &x,
                                           tiledb::Dimension *poBands)
@@ -541,17 +544,17 @@ CPLErr TileDBRasterDataset::AddDimensions(tiledb::Domain &domain,
     {
         case ATTRIBUTES:
             domain.add_dimensions(y, x);
-            CreateAttribute(eDataType, TILEDB_VALUES, nBands);
+            CreateAttribute(eDataType, pszAttrName, nBands);
             break;
         case PIXEL:
             assert(poBands);
             domain.add_dimensions(y, x, *poBands);
-            CreateAttribute(eDataType, TILEDB_VALUES, 1);
+            CreateAttribute(eDataType, pszAttrName, 1);
             break;
         default:  // BAND
             assert(poBands);
             domain.add_dimensions(*poBands, y, x);
-            CreateAttribute(eDataType, TILEDB_VALUES, 1);
+            CreateAttribute(eDataType, pszAttrName, 1);
             break;
     }
 
@@ -610,11 +613,7 @@ CPLErr TileDBRasterDataset::TrySaveXML()
         if (psTree == nullptr)
         {
             /* If we have unset all metadata, we have to delete the PAM file */
-#if TILEDB_VERSION_MAJOR == 1 && TILEDB_VERSION_MINOR < 7
-            vfs.remove_file(psPam->pszPamFilename);
-#else
-            m_array->delete_metadata("_gdal");
-#endif
+            m_array->delete_metadata(GDAL_ATTRIBUTE_NAME);
             return CE_None;
         }
 
@@ -684,18 +683,17 @@ CPLErr TileDBRasterDataset::TrySaveXML()
         /*      Try saving the auxiliary metadata. */
         /* --------------------------------------------------------------------
          */
-        bool bSaved = false;
         CPLErrorHandlerPusher oQuietError(CPLQuietErrorHandler);
         char *pszTree = CPLSerializeXMLTree(psTree);
 
-#if TILEDB_VERSION_MAJOR > 1 || TILEDB_VERSION_MINOR >= 7
         if (eAccess == GA_ReadOnly)
         {
             if (nTimestamp)
             {
                 auto oMeta = std::unique_ptr<tiledb::Array>(new tiledb::Array(
-                    *m_ctx, m_array->uri(), TILEDB_WRITE, nTimestamp));
-                oMeta->put_metadata("_gdal", TILEDB_UINT8,
+                    *m_ctx, m_array->uri(), TILEDB_WRITE,
+                    tiledb::TemporalPolicy(tiledb::TimeTravel, nTimestamp)));
+                oMeta->put_metadata(GDAL_ATTRIBUTE_NAME, TILEDB_UINT8,
                                     static_cast<int>(strlen(pszTree)), pszTree);
                 oMeta->close();
             }
@@ -703,75 +701,22 @@ CPLErr TileDBRasterDataset::TrySaveXML()
             {
                 auto oMeta = std::unique_ptr<tiledb::Array>(
                     new tiledb::Array(*m_ctx, m_array->uri(), TILEDB_WRITE));
-                oMeta->put_metadata("_gdal", TILEDB_UINT8,
+                oMeta->put_metadata(GDAL_ATTRIBUTE_NAME, TILEDB_UINT8,
                                     static_cast<int>(strlen(pszTree)), pszTree);
                 oMeta->close();
             }
         }
         else
         {
-            m_array->put_metadata("_gdal", TILEDB_UINT8,
+            m_array->put_metadata("dataset_type", TILEDB_STRING_UTF8,
+                                  static_cast<int>(strlen(RASTER_DATASET_TYPE)),
+                                  RASTER_DATASET_TYPE);
+            m_array->put_metadata(GDAL_ATTRIBUTE_NAME, TILEDB_UINT8,
                                   static_cast<int>(strlen(pszTree)), pszTree);
         }
 
-        bSaved = true;
-#endif
-
-        // cppcheck-suppress knownConditionTrueFalse
-        if (!bSaved)
-        {
-            vfs.touch(psPam->pszPamFilename);
-            tiledb::VFS::filebuf fbuf(vfs);
-            fbuf.open(psPam->pszPamFilename, std::ios::out);
-            std::ostream os(&fbuf);
-
-            if (os.good())
-            {
-                os.write(pszTree, strlen(pszTree));
-                bSaved = true;
-            }
-
-            fbuf.close();
-        }
-
         CPLFree(pszTree);
 
-        /* --------------------------------------------------------------------
-         */
-        /*      If it fails, check if we have a proxy directory for auxiliary */
-        /*      metadata to be stored in, and try to save there. */
-        /* --------------------------------------------------------------------
-         */
-        CPLErr eErr = CE_None;
-
-        if (bSaved)
-            eErr = CE_None;
-        else
-        {
-            const char *pszBasename = GetDescription();
-
-            if (psPam->osPhysicalFilename.length() > 0)
-                pszBasename = psPam->osPhysicalFilename;
-
-            const char *pszNewPam = nullptr;
-            if (PamGetProxy(pszBasename) == nullptr &&
-                ((pszNewPam = PamAllocateProxy(pszBasename)) != nullptr))
-            {
-                CPLErrorReset();
-                CPLFree(psPam->pszPamFilename);
-                psPam->pszPamFilename = CPLStrdup(pszNewPam);
-                eErr = TrySaveXML();
-            }
-            /* No way we can save into a /vsicurl resource */
-            else if (!VSISupportsSequentialWrite(psPam->pszPamFilename, false))
-            {
-                CPLError(CE_Warning, CPLE_AppDefined,
-                         "Unable to save auxiliary information in %s.",
-                         psPam->pszPamFilename);
-                eErr = CE_Warning;
-            }
-        }
-
         /* --------------------------------------------------------------------
          */
         /*      Cleanup */
@@ -780,7 +725,7 @@ CPLErr TileDBRasterDataset::TrySaveXML()
         if (psTree)
             CPLDestroyXMLNode(psTree);
 
-        return eErr;
+        return CE_None;
     }
     catch (const std::exception &e)
     {
@@ -850,7 +795,6 @@ CPLErr TileDBRasterDataset::TryLoadCachedXML(char ** /*papszSiblingFiles*/,
         {
             CPLErrorHandlerPusher oQuietError(CPLQuietErrorHandler);
 
-#if TILEDB_VERSION_MAJOR > 1 || TILEDB_VERSION_MINOR >= 7
             if (bReload)
             {
                 tiledb_datatype_t v_type =
@@ -859,7 +803,8 @@ CPLErr TileDBRasterDataset::TryLoadCachedXML(char ** /*papszSiblingFiles*/,
                 uint32_t v_num = 0;
                 if ((eAccess == GA_Update) && (m_roArray))
                 {
-                    m_roArray->get_metadata("_gdal", &v_type, &v_num, &v_r);
+                    m_roArray->get_metadata(GDAL_ATTRIBUTE_NAME, &v_type,
+                                            &v_num, &v_r);
                     if (v_r)
                     {
                         osMetaDoc =
@@ -868,7 +813,8 @@ CPLErr TileDBRasterDataset::TryLoadCachedXML(char ** /*papszSiblingFiles*/,
                 }
                 else
                 {
-                    m_array->get_metadata("_gdal", &v_type, &v_num, &v_r);
+                    m_array->get_metadata(GDAL_ATTRIBUTE_NAME, &v_type, &v_num,
+                                          &v_r);
 
                     if (v_r)
                     {
@@ -878,7 +824,7 @@ CPLErr TileDBRasterDataset::TryLoadCachedXML(char ** /*papszSiblingFiles*/,
                 }
                 psTree = CPLParseXMLString(osMetaDoc);
             }
-#endif
+
             if (bReload && psTree == nullptr &&
                 vfs.is_file(psPam->pszPamFilename))
             {
@@ -1004,7 +950,7 @@ char **TileDBRasterDataset::GetMetadata(const char *pszDomain)
 GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
 
 {
-    auto poDS = cpl::make_unique<TileDBRasterDataset>();
+    auto poDS = std::make_unique<TileDBRasterDataset>();
 
     const char *pszConfig =
         CSLFetchNameValue(poOpenInfo->papszOpenOptions, "TILEDB_CONFIG");
@@ -1031,6 +977,7 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
     CPLString osAux;
     CPLString osSubdataset;
 
+    std::string osAttrNameTmp;
     const char *pszAttr =
         CSLFetchNameValue(poOpenInfo->papszOpenOptions, "TILEDB_ATTRIBUTE");
 
@@ -1084,12 +1031,14 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
         if (eMode == TILEDB_READ)
         {
             poDS->m_array.reset(new tiledb::Array(
-                *poDS->m_ctx, osArrayPath, TILEDB_READ, poDS->nTimestamp));
+                *poDS->m_ctx, osArrayPath, TILEDB_READ,
+                tiledb::TemporalPolicy(tiledb::TimeTravel, poDS->nTimestamp)));
         }
         else
         {
             poDS->m_array.reset(new tiledb::Array(
-                *poDS->m_ctx, osArrayPath, TILEDB_WRITE, poDS->nTimestamp));
+                *poDS->m_ctx, osArrayPath, TILEDB_WRITE,
+                tiledb::TemporalPolicy(tiledb::TimeTravel, poDS->nTimestamp)));
         }
     }
     else
@@ -1159,6 +1108,17 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
             poDS->eDataType = eDT;
         }
     }
+    else
+    {
+        if (!pszAttr)
+        {
+            if (schema.attribute_num() == 1)
+            {
+                osAttrNameTmp = schema.attribute(0).name();
+                pszAttr = osAttrNameTmp.c_str();
+            }
+        }
+    }
 
     const char *pszIndexMode = CSLFetchNameValue(papszStructMeta, "INTERLEAVE");
 
@@ -1167,6 +1127,8 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
 
     std::vector<tiledb::Dimension> dims = schema.domain().dimensions();
 
+    int iYDim = 0;
+    int iXDim = 1;
     if ((dims.size() == 2) || (dims.size() == 3))
     {
         if (dims.size() == 3)
@@ -1183,15 +1145,27 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
             if (poDS->eIndexMode == PIXEL)
                 std::rotate(dims.begin(), dims.begin() + 2, dims.end());
 
-            poDS->nBandStart =
-                static_cast<int>(dims[0].domain<uint64_t>().first);
-            poDS->nBands =
-                static_cast<int>(dims[0].domain<uint64_t>().second -
-                                 dims[0].domain<uint64_t>().first + 1);
-            poDS->nBlockYSize =
-                static_cast<int>(dims[1].tile_extent<uint64_t>());
-            poDS->nBlockXSize =
-                static_cast<int>(dims[2].tile_extent<uint64_t>());
+            if (dims[0].type() != TILEDB_UINT64)
+            {
+                const char *pszTypeName = "";
+                tiledb_datatype_to_str(dims[0].type(), &pszTypeName);
+                CPLError(CE_Failure, CPLE_NotSupported,
+                         "Unsupported BAND dimension type: %s", pszTypeName);
+                return nullptr;
+            }
+            poDS->nBandStart = dims[0].domain<uint64_t>().first;
+            const uint64_t nBandEnd = dims[0].domain<uint64_t>().second;
+            if (nBandEnd < poDS->nBandStart ||
+                nBandEnd - poDS->nBandStart >
+                    static_cast<uint64_t>(std::numeric_limits<int>::max() - 1))
+            {
+                CPLError(CE_Failure, CPLE_NotSupported,
+                         "Invalid bounds for BAND dimension.");
+                return nullptr;
+            }
+            poDS->nBands = static_cast<int>(nBandEnd - poDS->nBandStart + 1);
+            iYDim = 1;
+            iXDim = 2;
         }
         else
         {
@@ -1200,10 +1174,6 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
             if (pszBands)
             {
                 poDS->nBands = atoi(pszBands);
-                poDS->nBlockYSize =
-                    static_cast<int>(dims[0].tile_extent<uint64_t>());
-                poDS->nBlockXSize =
-                    static_cast<int>(dims[1].tile_extent<uint64_t>());
             }
 
             poDS->eIndexMode = ATTRIBUTES;
@@ -1217,6 +1187,70 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
         return nullptr;
     }
 
+    if (!GDALCheckDatasetDimensions(poDS->nRasterXSize, poDS->nRasterYSize) ||
+        !GDALCheckBandCount(poDS->nBands, /*bIsZeroAllowed=*/true))
+    {
+        return nullptr;
+    }
+
+    if (dims[iYDim].type() != TILEDB_UINT64)
+    {
+        const char *pszTypeName = "";
+        tiledb_datatype_to_str(dims[0].type(), &pszTypeName);
+        CPLError(CE_Failure, CPLE_NotSupported,
+                 "Unsupported Y dimension type: %s", pszTypeName);
+        return nullptr;
+    }
+    if (!pszYSize)
+    {
+        const uint64_t nStart = dims[iYDim].domain<uint64_t>().first;
+        const uint64_t nEnd = dims[iYDim].domain<uint64_t>().second;
+        if (nStart != 0 ||
+            nEnd > static_cast<uint64_t>(std::numeric_limits<int>::max() - 1))
+        {
+            CPLError(CE_Failure, CPLE_NotSupported,
+                     "Invalid bounds for Y dimension.");
+            return nullptr;
+        }
+        poDS->nRasterYSize = static_cast<int>(nEnd - nStart + 1);
+    }
+    const uint64_t nBlockYSize = dims[iYDim].tile_extent<uint64_t>();
+    if (nBlockYSize > static_cast<uint64_t>(std::numeric_limits<int>::max()))
+    {
+        CPLError(CE_Failure, CPLE_NotSupported, "Too large block Y size.");
+        return nullptr;
+    }
+    poDS->nBlockYSize = static_cast<int>(nBlockYSize);
+
+    if (dims[iXDim].type() != TILEDB_UINT64)
+    {
+        const char *pszTypeName = "";
+        tiledb_datatype_to_str(dims[0].type(), &pszTypeName);
+        CPLError(CE_Failure, CPLE_NotSupported,
+                 "Unsupported Y dimension type: %s", pszTypeName);
+        return nullptr;
+    }
+    if (!pszXSize)
+    {
+        const uint64_t nStart = dims[iXDim].domain<uint64_t>().first;
+        const uint64_t nEnd = dims[iXDim].domain<uint64_t>().second;
+        if (nStart != 0 ||
+            nEnd > static_cast<uint64_t>(std::numeric_limits<int>::max() - 1))
+        {
+            CPLError(CE_Failure, CPLE_NotSupported,
+                     "Invalid bounds for X dimension.");
+            return nullptr;
+        }
+        poDS->nRasterXSize = static_cast<int>(nEnd - nStart + 1);
+    }
+    const uint64_t nBlockXSize = dims[iXDim].tile_extent<uint64_t>();
+    if (nBlockXSize > static_cast<uint64_t>(std::numeric_limits<int>::max()))
+    {
+        CPLError(CE_Failure, CPLE_NotSupported, "Too large block X size.");
+        return nullptr;
+    }
+    poDS->nBlockXSize = static_cast<int>(nBlockXSize);
+
     poDS->nBlocksX = DIV_ROUND_UP(poDS->nRasterXSize, poDS->nBlockXSize);
     poDS->nBlocksY = DIV_ROUND_UP(poDS->nRasterYSize, poDS->nBlockYSize);
 
@@ -1289,16 +1323,12 @@ GDALDataset *TileDBRasterDataset::Open(GDALOpenInfo *poOpenInfo)
             else if (poDS->eIndexMode == ATTRIBUTES)
             {
                 poDS->nBands = schema.attribute_num();
-                poDS->nBlockYSize =
-                    static_cast<int>(dims[0].tile_extent<uint64_t>());
-                poDS->nBlockXSize =
-                    static_cast<int>(dims[1].tile_extent<uint64_t>());
 
                 // Create band information objects.
                 for (int i = 1; i <= poDS->nBands; ++i)
                 {
                     CPLString osAttr =
-                        TILEDB_VALUES + CPLString().Printf("_%i", i + 1);
+                        TILEDB_VALUES + CPLString().Printf("_%i", i);
                     poDS->SetBand(i,
                                   new TileDBRasterBand(poDS.get(), i, osAttr));
                 }
@@ -1367,6 +1397,13 @@ CPLErr TileDBRasterDataset::CreateAttribute(GDALDataType eType,
                     nBitsPerSample = 8;
                     break;
                 }
+                case GDT_Int8:
+                {
+                    m_schema->add_attribute(tiledb::Attribute::create<int8_t>(
+                        *m_ctx, osName, *m_filterList));
+                    nBitsPerSample = 8;
+                    break;
+                }
                 case GDT_UInt16:
                 {
                     m_schema->add_attribute(
@@ -1383,6 +1420,13 @@ CPLErr TileDBRasterDataset::CreateAttribute(GDALDataType eType,
                     nBitsPerSample = 32;
                     break;
                 }
+                case GDT_UInt64:
+                {
+                    m_schema->add_attribute(tiledb::Attribute::create<uint64_t>(
+                        *m_ctx, osName, *m_filterList));
+                    nBitsPerSample = 64;
+                    break;
+                }
                 case GDT_Int16:
                 {
                     m_schema->add_attribute(tiledb::Attribute::create<short>(
@@ -1397,6 +1441,13 @@ CPLErr TileDBRasterDataset::CreateAttribute(GDALDataType eType,
                     nBitsPerSample = 32;
                     break;
                 }
+                case GDT_Int64:
+                {
+                    m_schema->add_attribute(tiledb::Attribute::create<int64_t>(
+                        *m_ctx, osName, *m_filterList));
+                    nBitsPerSample = 64;
+                    break;
+                }
                 case GDT_Float32:
                 {
                     m_schema->add_attribute(tiledb::Attribute::create<float>(
@@ -1587,7 +1638,7 @@ TileDBRasterDataset *TileDBRasterDataset::CreateLL(const char *pszFilename,
             return nullptr;
         }
 
-        auto poDS = cpl::make_unique<TileDBRasterDataset>();
+        auto poDS = std::make_unique<TileDBRasterDataset>();
         poDS->nRasterXSize = nXSize;
         poDS->nRasterYSize = nYSize;
         poDS->eDataType = eType;
@@ -1684,15 +1735,21 @@ TileDBRasterDataset *TileDBRasterDataset::CreateLL(const char *pszFilename,
         auto d2 = tiledb::Dimension::create<uint64_t>(
             *poDS->m_ctx, "Y", {0, h}, uint64_t(poDS->nBlockYSize));
 
-        if ((poDS->nBands == 0) || (poDS->eIndexMode == ATTRIBUTES))
         {
-            poDS->AddDimensions(domain, d2, d1, nullptr);
-        }
-        else
-        {
-            auto d3 = tiledb::Dimension::create<uint64_t>(
-                *poDS->m_ctx, "BANDS", {1, uint64_t(poDS->nBands)}, 1);
-            poDS->AddDimensions(domain, d2, d1, &d3);
+            // Only used for unit test purposes (to check ability of GDAL to read
+            // an arbitrary array)
+            const char *pszAttrName =
+                CPLGetConfigOption("TILEDB_ATTRIBUTE", TILEDB_VALUES);
+            if ((poDS->nBands == 0) || (poDS->eIndexMode == ATTRIBUTES))
+            {
+                poDS->AddDimensions(domain, pszAttrName, d2, d1, nullptr);
+            }
+            else
+            {
+                auto d3 = tiledb::Dimension::create<uint64_t>(
+                    *poDS->m_ctx, "BANDS", {1, uint64_t(poDS->nBands)}, 1);
+                poDS->AddDimensions(domain, pszAttrName, d2, d1, &d3);
+            }
         }
 
         poDS->m_schema->set_domain(domain).set_order(
@@ -1880,9 +1937,10 @@ CPLErr TileDBRasterDataset::CopySubDatasets(GDALDataset *poSrcDS,
 
         if (poDstDS->nTimestamp)
         {
-            poDstDS->m_array.reset(
-                new tiledb::Array(*poDstDS->m_ctx, poDstDS->GetDescription(),
-                                  TILEDB_WRITE, poDstDS->nTimestamp));
+            poDstDS->m_array.reset(new tiledb::Array(
+                *poDstDS->m_ctx, poDstDS->GetDescription(), TILEDB_WRITE,
+                tiledb::TemporalPolicy(tiledb::TimeTravel,
+                                       poDstDS->nTimestamp)));
         }
         else
             poDstDS->m_array.reset(new tiledb::Array(
@@ -1995,12 +2053,15 @@ GDALDataset *TileDBRasterDataset::Create(const char *pszFilename, int nXSize,
     tiledb::Array::create(osArrayPath, *poDS->m_schema);
 
     if (poDS->nTimestamp)
-        poDS->m_array.reset(new tiledb::Array(*poDS->m_ctx, osArrayPath,
-                                              TILEDB_WRITE, poDS->nTimestamp));
+        poDS->m_array.reset(new tiledb::Array(
+            *poDS->m_ctx, osArrayPath, TILEDB_WRITE,
+            tiledb::TemporalPolicy(tiledb::TimeTravel, poDS->nTimestamp)));
     else
         poDS->m_array.reset(
             new tiledb::Array(*poDS->m_ctx, osArrayPath, TILEDB_WRITE));
 
+    const char *pszAttrName =
+        CPLGetConfigOption("TILEDB_ATTRIBUTE", TILEDB_VALUES);
     for (int i = 0; i < poDS->nBands; i++)
     {
         if (poDS->eIndexMode == ATTRIBUTES)
@@ -2009,39 +2070,47 @@ GDALDataset *TileDBRasterDataset::Create(const char *pszFilename, int nXSize,
                            poDS.get(), i + 1,
                            TILEDB_VALUES + CPLString().Printf("_%i", i + 1)));
         else
-            poDS->SetBand(i + 1, new TileDBRasterBand(poDS.get(), i + 1));
+            poDS->SetBand(i + 1,
+                          new TileDBRasterBand(poDS.get(), i + 1, pszAttrName));
     }
 
-    char **papszImageStruct = nullptr;
-    papszImageStruct =
-        CSLAddNameValue(papszImageStruct, "NBITS",
-                        CPLString().Printf("%d", poDS->nBitsPerSample));
-    papszImageStruct = CSLAddNameValue(
-        papszImageStruct, "DATA_TYPE",
-        CPLString().Printf("%s", GDALGetDataTypeName(poDS->eDataType)));
-    papszImageStruct =
-        CSLAddNameValue(papszImageStruct, "X_SIZE",
-                        CPLString().Printf("%d", poDS->nRasterXSize));
-    papszImageStruct =
-        CSLAddNameValue(papszImageStruct, "Y_SIZE",
-                        CPLString().Printf("%d", poDS->nRasterYSize));
-    papszImageStruct = CSLAddNameValue(papszImageStruct, "INTERLEAVE",
-                                       index_type_name(poDS->eIndexMode));
-
-    if (poDS->lpoAttributeDS.size() > 0)
+    // Only used for unit test purposes (to check ability of GDAL to read
+    // an arbitrary array)
+    if (CPLTestBool(CPLGetConfigOption("TILEDB_WRITE_IMAGE_STRUCTURE", "YES")))
     {
-        int i = 0;
-        for (auto const &poAttrDS : poDS->lpoAttributeDS)
-        {
-            papszImageStruct =
-                CSLAddNameValue(papszImageStruct,
-                                CPLString().Printf("TILEDB_ATTRIBUTE_%i", ++i),
-                                CPLGetBasename(poAttrDS->GetDescription()));
-        }
-    }
-    poDS->SetMetadata(papszImageStruct, "IMAGE_STRUCTURE");
+        char **papszImageStruct = nullptr;
+        papszImageStruct =
+            CSLAddNameValue(papszImageStruct, "NBITS",
+                            CPLString().Printf("%d", poDS->nBitsPerSample));
+        papszImageStruct = CSLAddNameValue(
+            papszImageStruct, "DATA_TYPE",
+            CPLString().Printf("%s", GDALGetDataTypeName(poDS->eDataType)));
+        papszImageStruct =
+            CSLAddNameValue(papszImageStruct, "X_SIZE",
+                            CPLString().Printf("%d", poDS->nRasterXSize));
+        papszImageStruct =
+            CSLAddNameValue(papszImageStruct, "Y_SIZE",
+                            CPLString().Printf("%d", poDS->nRasterYSize));
+        papszImageStruct = CSLAddNameValue(papszImageStruct, "INTERLEAVE",
+                                           index_type_name(poDS->eIndexMode));
+        papszImageStruct = CSLAddNameValue(papszImageStruct, "DATASET_TYPE",
+                                           RASTER_DATASET_TYPE);
 
-    CSLDestroy(papszImageStruct);
+        if (poDS->lpoAttributeDS.size() > 0)
+        {
+            int i = 0;
+            for (auto const &poAttrDS : poDS->lpoAttributeDS)
+            {
+                papszImageStruct = CSLAddNameValue(
+                    papszImageStruct,
+                    CPLString().Printf("TILEDB_ATTRIBUTE_%i", ++i),
+                    CPLGetBasename(poAttrDS->GetDescription()));
+            }
+        }
+        poDS->SetMetadata(papszImageStruct, "IMAGE_STRUCTURE");
+
+        CSLDestroy(papszImageStruct);
+    }
 
     return poDS.release();
 }
@@ -2204,7 +2273,3 @@ GDALDataset *TileDBRasterDataset::CreateCopy(const char *pszFilename,
     }
     return nullptr;
 }
-
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
