@@ -49,6 +49,7 @@
 #include "ogr_geojson.h"
 #include "ogrgeojsonreader.h"
 #include "ogrgeojsonutils.h"
+
 // #include "symbol_renames.h"
 
 /************************************************************************/
@@ -158,11 +159,14 @@ void OGRESRIJSONReader::ReadLayers(OGRGeoJSONDataSource *poDS,
 bool OGRESRIJSONReader::GenerateLayerDefn()
 {
     CPLAssert(nullptr != poGJObject_);
-    CPLAssert(nullptr != poLayer_->GetLayerDefn());
-    CPLAssert(0 == poLayer_->GetLayerDefn()->GetFieldCount());
 
     bool bSuccess = true;
 
+    OGRFeatureDefn *poDefn = poLayer_->GetLayerDefn();
+    CPLAssert(nullptr != poDefn);
+    CPLAssert(0 == poDefn->GetFieldCount());
+    auto oTemporaryUnsealer(poDefn->GetTemporaryUnsealer());
+
     /* -------------------------------------------------------------------- */
     /*      Scan all features and generate layer definition.                */
     /* -------------------------------------------------------------------- */
@@ -187,7 +191,6 @@ bool OGRESRIJSONReader::GenerateLayerDefn()
         if (nullptr != poFields &&
             json_object_get_type(poFields) == json_type_object)
         {
-            OGRFeatureDefn *poDefn = poLayer_->GetLayerDefn();
             json_object_iter it;
             it.key = nullptr;
             it.val = nullptr;
@@ -924,7 +927,7 @@ OGRGeometry *OGRESRIJSONReadPolygon(json_object *poObj)
         }
 
         OGRPolygon *poPoly = new OGRPolygon();
-        auto poLine = cpl::make_unique<OGRLinearRing>();
+        auto poLine = std::make_unique<OGRLinearRing>();
         papoGeoms[iRing] = poPoly;
 
         const auto nPoints = json_object_array_length(poObjRing);
