@@ -1677,7 +1677,7 @@ OGRErr OGRGeoRSSLayer::ICreateFeature(OGRFeature *poFeatureIn)
 /*                            CreateField()                             */
 /************************************************************************/
 
-OGRErr OGRGeoRSSLayer::CreateField(OGRFieldDefn *poFieldDefn,
+OGRErr OGRGeoRSSLayer::CreateField(const OGRFieldDefn *poFieldDefn,
                                    CPL_UNUSED int bApproxOK)
 {
     const char *pszName = poFieldDefn->GetNameRef();
@@ -2296,3 +2296,12 @@ GIntBig OGRGeoRSSLayer::GetFeatureCount(int bForce)
 
     return nTotalFeatureCount;
 }
+
+/************************************************************************/
+/*                             GetDataset()                             */
+/************************************************************************/
+
+GDALDataset *OGRGeoRSSLayer::GetDataset()
+{
+    return poDS;
+}
