@@ -220,7 +220,7 @@ bool OGRGeoPackageLayer::ParseDateField(const char *pszTxt, OGRField *psField,
             bError = true;
         }
     }
-    else if (OGRParseDate(pszTxt, psField, 0))
+    else if (OGRParseDate(pszTxt, psField, OGRPARSEDATE_OPTION_LAX))
     {
         constexpr int line = __LINE__;
         if (!m_poDS->m_oSetGPKGLayerWarnings[line])
@@ -308,7 +308,7 @@ bool OGRGeoPackageLayer::ParseDateTimeField(const char *pszTxt,
         // nominal format is YYYYMMDDTHHMMSSsssZ before GeoPackage 1.4
         // GeoPackage 1.4 also accepts omission of seconds and milliseconds
     }
-    else if (OGRParseDate(pszTxt, psField, 0))
+    else if (OGRParseDate(pszTxt, psField, OGRPARSEDATE_OPTION_LAX))
     {
         constexpr int line = __LINE__;
         if (!m_poDS->m_oSetGPKGLayerWarnings[line])
@@ -397,8 +397,16 @@ OGRFeature *OGRGeoPackageLayer::TranslateFeature(sqlite3_stmt *hStmt)
                              "Unable to read geometry");
                 }
             }
-            if (poGeom != nullptr)
+            if (poGeom)
+            {
+                if (m_bUndoDiscardCoordLSBOnReading)
+                {
+                    poGeom->roundCoordinates(
+                        poGeomFieldDefn->GetCoordinatePrecision());
+                }
                 poGeom->assignSpatialReference(poSrs);
+            }
+
             poFeature->SetGeometryDirectly(poGeom);
         }
     }
@@ -633,7 +641,8 @@ int OGRGeoPackageLayer::GetNextArrowArray(struct ArrowArrayStream *stream,
                 const GByte *pabyGpkg = static_cast<const GByte *>(
                     sqlite3_column_blob(hStmt, m_iGeomCol));
                 if (m_poFilterGeom == nullptr && iGpkgSize >= 8 && pabyGpkg &&
-                    pabyGpkg[0] == 'G' && pabyGpkg[1] == 'P')
+                    pabyGpkg[0] == 'G' && pabyGpkg[1] == 'P' &&
+                    !m_bUndoDiscardCoordLSBOnReading)
                 {
                     GPkgHeader oHeader;
 
@@ -663,6 +672,12 @@ int OGRGeoPackageLayer::GetNextArrowArray(struct ArrowArrayStream *stream,
                         }
                         poGeom.reset(poGeomPtr);
                     }
+                    else if (m_bUndoDiscardCoordLSBOnReading)
+                    {
+                        poGeom->roundCoordinates(
+                            m_poFeatureDefn->GetGeomFieldDefn(0)
+                                ->GetCoordinatePrecision());
+                    }
                     if (poGeom != nullptr)
                     {
                         nWKBSize = poGeom->WkbSize();
@@ -1227,7 +1242,7 @@ void OGRGeoPackageLayer::BuildFeatureDefn(const char *pszLayerName,
 /*                          SetIgnoredFields()                          */
 /************************************************************************/
 
-OGRErr OGRGeoPackageLayer::SetIgnoredFields(const char **papszFields)
+OGRErr OGRGeoPackageLayer::SetIgnoredFields(CSLConstList papszFields)
 {
     OGRErr eErr = OGRLayer::SetIgnoredFields(papszFields);
     if (eErr == OGRERR_NONE)
