@@ -27,6 +27,9 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+#ifndef OGRLIBKMLFIELD_H_INCLUDED
+#define OGRLIBKMLFIELD_H_INCLUDED
+
 /******************************************************************************
  Function to output ogr fields in kml.
 
@@ -64,8 +67,9 @@ void kml2field(OGRFeature *poOgrFeat, kmldom::FeaturePtr poKmlFeature);
  Function create a simplefield from a FieldDefn.
 ******************************************************************************/
 
-kmldom::SimpleFieldPtr FieldDef2kml(OGRFieldDefn *poOgrFieldDef,
-                                    kmldom::KmlFactory *poKmlFactory);
+kmldom::SimpleFieldPtr FieldDef2kml(const OGRFieldDefn *poOgrFieldDef,
+                                    kmldom::KmlFactory *poKmlFactory,
+                                    bool bApproxOK);
 
 /******************************************************************************
  Function to add the simpleFields in a schema to a featuredefn.
@@ -129,3 +133,5 @@ struct fieldconfig
 void get_fieldconfig(struct fieldconfig *oFC);
 
 int kmlAltitudeModeFromString(const char *pszAltitudeMode, int &isGX);
+
+#endif /* OGRLIBKMLFIELD_H_INCLUDED */
