@@ -38,6 +38,8 @@
 #include "ogrpgutility.h"
 #include "ogr_pgdump.h"
 
+#include <map>
+#include <optional>
 #include <vector>
 
 /* These are the OIDs for some builtin types, as returned by PQftype(). */
@@ -194,7 +196,7 @@ class OGRPGLayer CPL_NON_FINAL : public OGRLayer
     static char *GeometryToBYTEA(const OGRGeometry *, int nPostGISMajor,
                                  int nPostGISMinor);
     static GByte *BYTEAToGByteArray(const char *pszBytea, int *pnLength);
-    static OGRGeometry *BYTEAToGeometry(const char *, int bIsPostGIS1);
+    static OGRGeometry *BYTEAToGeometry(const char *);
     Oid GeometryToOID(OGRGeometry *);
     OGRGeometry *OIDToGeometry(Oid);
 
@@ -222,8 +224,11 @@ class OGRPGLayer CPL_NON_FINAL : public OGRLayer
     void CloseCursor();
 
     virtual CPLString GetFromClauseForGetExtent() = 0;
-    OGRErr RunGetExtentRequest(OGREnvelope *psExtent, int bForce,
-                               CPLString osCommand, int bErrorAsDebug);
+    OGRErr RunGetExtentRequest(OGREnvelope &sExtent, int bForce,
+                               const std::string &osCommand, int bErrorAsDebug);
+    OGRErr RunGetExtent3DRequest(OGREnvelope3D &sExtent3D,
+                                 const std::string &osCommand,
+                                 int bErrorAsDebug);
     static void CreateMapFromFieldNameToIndex(PGresult *hResult,
                                               OGRFeatureDefn *poFeatureDefn,
                                               int *&panMapFieldNameToIndex,
@@ -252,9 +257,13 @@ class OGRPGLayer CPL_NON_FINAL : public OGRLayer
     {
         return GetExtent(0, psExtent, bForce);
     }
+
     virtual OGRErr GetExtent(int iGeomField, OGREnvelope *psExtent,
                              int bForce) override;
 
+    OGRErr GetExtent3D(int iGeomField, OGREnvelope3D *psExtent3D,
+                       int bForce) override;
+
     virtual OGRErr StartTransaction() override;
     virtual OGRErr CommitTransaction() override;
     virtual OGRErr RollbackTransaction() override;
@@ -270,6 +279,8 @@ class OGRPGLayer CPL_NON_FINAL : public OGRLayer
         return poDS;
     }
 
+    GDALDataset *GetDataset() override;
+
     virtual void ResolveSRID(const OGRPGGeomFieldDefn *poGFldDefn) = 0;
 };
 
@@ -294,6 +305,8 @@ class OGRPGTableLayer final : public OGRPGLayer
     char *pszSchemaName = nullptr;
     char *m_pszTableDescription = nullptr;
     CPLString osForcedDescription{};
+    bool m_bMetadataLoaded = false;
+    bool m_bMetadataModified = false;
     char *pszSqlTableName = nullptr;
     int bTableDefinitionValid = -1;
 
@@ -311,6 +324,7 @@ class OGRPGTableLayer final : public OGRPGLayer
     CPLString osWHERE{};
 
     int bLaunderColumnNames = true;
+    bool m_bUTF8ToASCII = false;
     int bPreservePrecision = true;
     int bUseCopy = USE_COPY_UNSET;
     int bCopyActive = false;
@@ -361,6 +375,9 @@ class OGRPGTableLayer final : public OGRPGLayer
 
     void UpdateSequenceIfNeeded();
 
+    void LoadMetadata();
+    void SerializeMetadata();
+
   public:
     OGRPGTableLayer(OGRPGDataSource *, CPLString &osCurrentSchema,
                     const char *pszTableName, const char *pszSchemaName,
@@ -379,6 +396,7 @@ class OGRPGTableLayer final : public OGRPGLayer
     {
         SetSpatialFilter(0, poGeom);
     }
+
     virtual void SetSpatialFilter(int iGeomField, OGRGeometry *poGeom) override;
 
     virtual OGRErr SetAttributeFilter(const char *) override;
@@ -392,9 +410,9 @@ class OGRPGTableLayer final : public OGRPGLayer
     virtual OGRErr DeleteFeature(GIntBig nFID) override;
     virtual OGRErr ICreateFeature(OGRFeature *poFeature) override;
 
-    virtual OGRErr CreateField(OGRFieldDefn *poField,
+    virtual OGRErr CreateField(const OGRFieldDefn *poField,
                                int bApproxOK = TRUE) override;
-    virtual OGRErr CreateGeomField(OGRGeomFieldDefn *poGeomField,
+    virtual OGRErr CreateGeomField(const OGRGeomFieldDefn *poGeomField,
                                    int bApproxOK = TRUE) override;
     virtual OGRErr DeleteField(int iField) override;
     virtual OGRErr AlterFieldDefn(int iField, OGRFieldDefn *poNewFieldDefn,
@@ -410,6 +428,7 @@ class OGRPGTableLayer final : public OGRPGLayer
     {
         return GetExtent(0, psExtent, bForce);
     }
+
     virtual OGRErr GetExtent(int iGeomField, OGREnvelope *psExtent,
                              int bForce) override;
 
@@ -417,6 +436,7 @@ class OGRPGTableLayer final : public OGRPGLayer
     {
         return pszTableName;
     }
+
     const char *GetSchemaName()
     {
         return pszSchemaName;
@@ -445,6 +465,12 @@ class OGRPGTableLayer final : public OGRPGLayer
     {
         bLaunderColumnNames = bFlag;
     }
+
+    void SetUTF8ToASCIIFlag(bool bFlag)
+    {
+        m_bUTF8ToASCII = bFlag;
+    }
+
     void SetPrecisionFlag(int bFlag)
     {
         bPreservePrecision = bFlag;
@@ -456,10 +482,12 @@ class OGRPGTableLayer final : public OGRPGLayer
     OGRErr EndCopy();
 
     int ReadTableDefinition();
+
     int HasGeometryInformation()
     {
         return bGeometryInformationSet;
     }
+
     void SetTableDefinition(const char *pszFIDColumnName,
                             const char *pszGFldName, OGRwkbGeometryType eType,
                             const char *pszGeomType, int nSRSId,
@@ -469,27 +497,33 @@ class OGRPGTableLayer final : public OGRPGLayer
     {
         nForcedSRSId = nForcedSRSIdIn;
     }
+
     void SetForcedGeometryTypeFlags(int GeometryTypeFlagsIn)
     {
         nForcedGeometryTypeFlags = GeometryTypeFlagsIn;
     }
+
     void SetCreateSpatialIndex(bool bFlag, const char *pszSpatialIndexType)
     {
         bCreateSpatialIndexFlag = bFlag;
         osSpatialIndexType = pszSpatialIndexType;
     }
+
     void SetForcedDescription(const char *pszDescriptionIn);
+
     void AllowAutoFIDOnCreateViaCopy()
     {
         bAutoFIDOnCreateViaCopy = TRUE;
     }
+
     void SetUseCopy()
     {
         bUseCopy = TRUE;
         bUseCopyByDefault = TRUE;
     }
 
-    void SetDeferredCreation(int bDeferredCreationIn, CPLString osCreateTable);
+    void SetDeferredCreation(int bDeferredCreationIn,
+                             const std::string &osCreateTable);
     OGRErr RunDeferredCreationIfNecessary();
 
     virtual void ResolveSRID(const OGRPGGeomFieldDefn *poGFldDefn) override;
@@ -533,6 +567,7 @@ class OGRPGResultLayer final : public OGRPGLayer
     {
         SetSpatialFilter(0, poGeom);
     }
+
     virtual void SetSpatialFilter(int iGeomField, OGRGeometry *poGeom) override;
 
     virtual int TestCapability(const char *) override;
@@ -569,7 +604,7 @@ class OGRPGDataSource final : public OGRDataSource
     int bHavePostGIS = false;
     int bHaveGeography = false;
 
-    int bUserTransactionActive = false;
+    bool bUserTransactionActive = false;
     int bSavePointActive = false;
     int nSoftTransactionLevel = 0;
 
@@ -582,9 +617,9 @@ class OGRPGDataSource final : public OGRDataSource
 
     // We maintain a list of known SRID to reduce the number of trips to
     // the database to get SRSes.
-    int nKnownSRID = 0;
-    int *panSRID = nullptr;
-    OGRSpatialReference **papoSRS = nullptr;
+    std::map<int,
+             std::unique_ptr<OGRSpatialReference, OGRSpatialReferenceReleaser>>
+        m_oSRSCache{};
 
     OGRPGTableLayer *poLayerInCopyMode = nullptr;
 
@@ -603,6 +638,9 @@ class OGRPGDataSource final : public OGRDataSource
     int bListAllTables = false;
     bool m_bSkipViews = false;
 
+    bool m_bOgrSystemTablesMetadataTableExistenceTested = false;
+    bool m_bOgrSystemTablesMetadataTableFound = false;
+
     void LoadTables();
 
     CPLString osDebugLastTransactionCommand{};
@@ -612,6 +650,8 @@ class OGRPGDataSource final : public OGRDataSource
 
     OGRErr FlushCacheWithRet(bool bAtClosing);
 
+    std::optional<std::string> FindSchema(const char *pszSchemaNameIn);
+
   public:
     PGver sPostgreSQLVersion = {0, 0, 0};
     PGver sPostGISVersion = {0, 0, 0};
@@ -622,10 +662,16 @@ class OGRPGDataSource final : public OGRDataSource
     bool m_bHasGeometryColumns = false;
     bool m_bHasSpatialRefSys = false;
 
+    bool HavePostGIS() const
+    {
+        return bHavePostGIS;
+    }
+
     int GetUndefinedSRID() const
     {
         return nUndefinedSRID;
     }
+
     bool IsUTF8ClientEncoding() const
     {
         return m_bUTF8ClientEncoding;
@@ -641,7 +687,7 @@ class OGRPGDataSource final : public OGRDataSource
     }
 
     int FetchSRSId(const OGRSpatialReference *poSRS);
-    OGRSpatialReference *FetchSRS(int nSRSId);
+    const OGRSpatialReference *FetchSRS(int nSRSId);
     static OGRErr InitializeMetadataTables();
 
     int Open(const char *, int bUpdate, int bTestOpen, char **papszOpenOptions);
@@ -654,16 +700,16 @@ class OGRPGDataSource final : public OGRDataSource
     {
         return pszName;
     }
+
     int GetLayerCount() override;
     OGRLayer *GetLayer(int) override;
     OGRLayer *GetLayerByName(const char *pszName) override;
 
     virtual CPLErr FlushCache(bool bAtClosing) override;
 
-    virtual OGRLayer *ICreateLayer(const char *,
-                                   OGRSpatialReference * = nullptr,
-                                   OGRwkbGeometryType = wkbUnknown,
-                                   char ** = nullptr) override;
+    OGRLayer *ICreateLayer(const char *pszName,
+                           const OGRGeomFieldDefn *poGeomFieldDefn,
+                           CSLConstList papszOptions) override;
 
     int TestCapability(const char *) override;
 
@@ -679,6 +725,7 @@ class OGRPGDataSource final : public OGRDataSource
     {
         return nGeometryOID;
     }
+
     Oid GetGeographyOID()
     {
         return nGeographyOID;
@@ -696,6 +743,14 @@ class OGRPGDataSource final : public OGRDataSource
     int UseCopy();
     void StartCopy(OGRPGTableLayer *poPGLayer);
     OGRErr EndCopy();
+
+    bool IsUserTransactionActive()
+    {
+        return bUserTransactionActive;
+    }
+
+    void CreateOgrSystemTablesMetadataTableIfNeeded();
+    bool HasOgrSystemTablesMetadataTable();
 };
 
 #endif /* ndef OGR_PG_H_INCLUDED */
